/*
 * Copyright (C) The Apache Software Foundation. All rights reserved.
 *
 * This software is published under the terms of the Apache Software License
 * version 1.1, a copy of which has been included with this distribution in
 * the LICENSE file.
 */
package org.apache.avalon.excalibur.pool;

import java.util.List;
import java.util.ArrayList;
import org.apache.avalon.excalibur.concurrent.Lock;
import org.apache.avalon.framework.activity.Initializable;
import org.apache.avalon.framework.logger.AbstractLoggable;
import org.apache.avalon.framework.thread.ThreadSafe;

/**
 * This is an <code>Pool</code> that caches Poolable objects for reuse.
 *
 * @author <a href="mailto:bloritsch@apache.org">Berin Loritsch</a>
 * @version CVS $Revision: 1.5 $ $Date: 2001/06/01 21:09:28 $
 */
public abstract class AbstractPool
    extends AbstractLoggable
    implements Pool, ThreadSafe
{
    public static final int        DEFAULT_POOL_SIZE  = 8;
    protected final ObjectFactory  m_factory;
    protected List                 m_active           = new ArrayList();
    protected List                 m_ready            = new ArrayList();
    protected Lock                 m_mutex            = new Lock();
    protected boolean              m_initialized      = false;

    /**
     * Create an AbstractPool.  The pool requires a factory, and can
     * optionally have a controller.
     */
    public AbstractPool( final ObjectFactory factory ) throws Exception
    {
        m_factory = factory;

        if( !(this instanceof Initializable) )
        {
            initialize();
        }
    }

    protected void initialize()
        throws Exception
    {
        m_mutex.lock();

        for( int i = 0; i < AbstractPool.DEFAULT_POOL_SIZE; i++ )
        {
            this.m_ready.add(this.newPoolable());
        }

        m_initialized = true;

        m_mutex.unlock();
    }


    /**
     * This is the method to override when you need to enforce creational
     * policies.
     */
    protected Poolable newPoolable() throws Exception
    {
        Object obj = m_factory.newInstance();
        return (Poolable)obj;
    }

    /**
     * This is the method to override when you need to enforce destructional
     * policies.
     */
    protected void removePoolable(Poolable poolable)
    {
        try
        {
            m_factory.decommission(poolable);
        }
        catch (Exception e)
        {
            getLogger().debug("Error decommissioning object", e);
        }
    }

    protected final int size() {
        synchronized(this)
        {
            return this.m_active.size() + this.m_ready.size();
        }
    }

    public abstract Poolable get() throws Exception;

    public abstract void put( Poolable object );
}
