/*
 * Copyright (C) The Apache Software Foundation. All rights reserved.
 *
 * This software is published under the terms of the Apache Software License
 * version 1.1, a copy of which has been included with this distribution in
 * the LICENSE file.
 */
package org.apache.avalon.excalibur.pool;

/**
 * This is an <code>Pool</code> that caches Poolable objects for reuse.
 *
 * @author <a href="mailto:bloritsch@apache.org">Berin Loritsch</a>
 * @version CVS $Revision: 1.6 $ $Date: 2001/05/08 15:13:33 $
 */
public class SoftResourceLimitingPool
    extends DefaultPool
    implements Resizable
{
    /**
     * Create an SoftResourceLimitingPool.  The pool requires a factory.
     */
    public SoftResourceLimitingPool( final ObjectFactory factory)
        throws Exception
    {
        this( factory, AbstractPool.DEFAULT_POOL_SIZE / 2);
    }

    /**
     * Create an SoftResourceLimitingPool.  The pool requires a factory,
     * and can optionally have a controller.
     */
    public SoftResourceLimitingPool( final ObjectFactory factory,
                                     final int min )
        throws Exception
    {
        this( factory, null, min, min * 2 );
    }

    /**
     * Create an SoftResourceLimitingPool.  The pool requires a factory,
     * and can optionally have a controller.
     */
    public SoftResourceLimitingPool( final ObjectFactory factory,
                                     final int min,
                                     final int max ) throws Exception
    {
        this( factory, null, min, max );
    }

    /**
     * Create an SoftResourceLimitingPool.  The pool requires a factory,
     * and can optionally have a controller.
     */
    public SoftResourceLimitingPool( final ObjectFactory factory,
                                     final PoolController controller,
                                     final int min,
                                     final int max )
        throws Exception
    {
        super( factory, controller, min, max );
    }

    public void initialize()
        throws Exception
    {
        this.grow(this.m_min);

        this.m_initialized = true;
    }

    public void grow( final int amount )
    {
        try
        {
            m_mutex.lock();

            for( int i = 0; i < amount; i++ )
            {
                try
                {
                    m_ready.add( newPoolable() );
                }
                catch( final Exception e )
                {
                    if( null != getLogger() )
                    {
                        getLogger().debug( m_factory.getCreatedClass().getName() +
                                           ": could not be instantiated.", e );
                    }
                }
            }
        }
        catch( final InterruptedException ie )
        {
            getLogger().warn( "Interrupted while waiting on lock", ie );
        }
        finally
        {
            m_mutex.unlock();
        }
    }

    public void shrink( final int amount )
    {
        try
        {
            m_mutex.lock();

            for( int i = 0; i < amount; i++ )
            {
                if( m_ready.size() > m_min )
                {
                    try
                    {
                        this.removePoolable( (Poolable) m_ready.remove( 0 ) );
                    }
                    catch( final Exception e )
                    {
                        if( null != getLogger() )
                        {
                            getLogger().debug( m_factory.getCreatedClass().getName() +
                                               ": improperly decommissioned.", e );
                        }
                    }
                }
            }
        }
        catch( final InterruptedException ie )
        {
            getLogger().warn("Interrupted while waiting on lock", ie);
        }
        finally
        {
            m_mutex.unlock();
        }
    }
}
