/*
 * Copyright (C) The Apache Software Foundation. All rights reserved.
 *
 * This software is published under the terms of the Apache Software License
 * version 1.1, a copy of which has been included with this distribution in
 * the LICENSE file.
 */
package org.apache.avalon.excalibur.i18n;

/** JDK classes **/
import java.io.IOException;
import java.util.Hashtable;
import java.util.HashMap;
import java.util.Locale;
import java.util.MissingResourceException;

/** W3C DOM classes **/
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.NamedNodeMap;

/** Xerces classes **/
import org.apache.xerces.dom.DocumentImpl;
import org.apache.xerces.dom.TextImpl;
import org.apache.xerces.parsers.DOMParser;
import org.xml.sax.SAXException;

/** Xalan classes **/
import org.apache.xalan.xpath.XPathSupport;
import org.apache.xalan.xpath.XPath;
import org.apache.xalan.xpath.XPathProcessorImpl;
import org.apache.xalan.xpath.xml.XMLParserLiaisonDefault;
import org.apache.xalan.xpath.xml.PrefixResolverDefault;
import org.apache.xalan.xpath.XObject;

/** Avalon classes **/
import org.apache.avalon.framework.logger.AbstractLoggable;

/**
 * @author <a href="mailto:mengelhart@earthtrip.com">Mike Engelhart</a>
 * @author <a href="mailto:neeme@one.lv">Neeme Praks</a>
 * @author <a href="mailto:oleg@one.lv">Oleg Podolsky</a>
 * @version $Id: XMLResourceBundle.java,v 1.2 2001/05/01 13:41:42 bloritsch Exp $
 */
public class XMLResourceBundle extends AbstractLoggable
{
    // Cache for storing string values for existing XPaths
    private Hashtable cache = new Hashtable();
    // Cache for storing non-existing XPaths
    private Hashtable cacheNotFound = new Hashtable();

    private String name = "";
    private Document doc;
    private Locale loc;
    protected XMLResourceBundle parent = null;

    public XMLResourceBundle(String name, Document doc, Locale loc, XMLResourceBundle p, boolean cacheAtStartup)
    {
        getLogger().info("Constructing XMLResourceBundle: " + name + ", locale: " + loc);
        this.name = name;
        this.doc = doc;
        this.loc = loc;
        this.parent = p;
        if (cacheAtStartup)
            cacheAll(doc.getDocumentElement(), "");
    }

    public XMLResourceBundle(String name, String fileName, Locale loc, XMLResourceBundle p, boolean cacheAtStartup) throws Exception
    {
        this(name, loadResourceBundle(fileName), loc, p, cacheAtStartup);
    }

    // Load the XML document based on bundleName
    protected static Document loadResourceBundle(String bundleName) throws Exception
    {
        DOMParser parser = new DOMParser();
        parser.parse(bundleName);
        return parser.getDocument();
    }

    public String getName()
    {
        return this.name;
    }

    public Document getDocument()
    {
        return this.doc;
    }

    public Locale getLocale()
    {
        return loc;
    }

    private boolean cacheContains(String key)
    {
        //if (logger.isDebugEnabled()) logger.debug(name + ": cache contains: " + key);
        return cache.containsKey(key);
    }

    private boolean cacheNotFoundContains(String key)
    {
        //if (logger.isDebugEnabled()) logger.debug(name + ": cache_not_found contains: " + key);
        return cacheNotFound.containsKey(key);
    }

    private void cacheKey(String key, String value)
    {
        //if (logger.isDebugEnabled()) logger.debug(name + ": caching: " + key + " = " + value);
        cache.put(key, value);
    }

    private void cacheNotFoundKey(String key)
    {
        getLogger().debug(name + ": caching not_found: " + key);
        cacheNotFound.put(key, "");
    }

    private String getFromCache(String key)
    {
        //if (logger.isDebugEnabled()) logger.debug(name + ": returning from cache: " + key);
        return (String) cache.get(key);
    }

    // Steps through the bundle tree and stores all text element values
    // in bundle's cache, and also stores attributes for all element nodes.
    // Parent must be am element-type node.
    private void cacheAll(Node parent, String pathToParent)
    {
        NodeList children = parent.getChildNodes();
        int childnum = children.getLength();

        for(int i = 0; i < childnum; i++)
        {
            Node child = children.item(i);

            if(child.getNodeType() == Node.ELEMENT_NODE)
            {
                String pathToChild = pathToParent + '/' + child.getNodeName();

                NamedNodeMap attrs = child.getAttributes();
                if(attrs != null)
                {
                    Node temp = null;
                    String pathToAttr = null;
                    int attrnum = attrs.getLength();
                    for(int j = 0; j < attrnum; j++)
                    {
                        temp = attrs.item(j);
                        if (!temp.getNodeName().equalsIgnoreCase("xml:lang"))
                            pathToChild += "[@" + temp.getNodeName() + "='" + temp.getNodeValue() + "']";
                    }
                }

                String childValue = getTextValue(child);
                if(childValue != null)
                    cacheKey(pathToChild, childValue);
                else
                    cacheAll(child, pathToChild);
            }
        }
    }

    // gets string without throwing an exception, returns empty string instead
    public String getStringSimple(String key)
    {
        String result = _getString(key);
        return result == null ? "" : result;
    }

    public String getString(String key) throws MissingResourceException
    {
        return getString(key, null);
    }

    public String getString(String key, HashMap dictionary) throws MissingResourceException
    {
        String value = _getString(key);
        if (value == null)
            throw new MissingResourceException("Unable to locate resource: " + key, XMLResourceBundle.class.getName(), key);
        else
            return substitute(value, dictionary);
    }

    private String _getString(String key)
    {
        if (key == null) return null;
        String value = getFromCache(key);

        if (value == null && !cacheNotFoundContains(key))
        {
            value = _getString(this.doc.getDocumentElement(), key);
            if (value == null)
            {
                if (this.parent != null)
                    value = this.parent._getString(key);
            }
            if (value != null)
                cacheKey(key, value);
            else
                cacheNotFoundKey(key);
        }
        return value;
    }

    public String getString(Node rootNode, String key) throws MissingResourceException
    {
        String value = _getString(rootNode, key);
        if (value == null)
            throw new MissingResourceException("Unable to locate resource: " + key, XMLResourceBundle.class.getName(), key);
        else
            return value;
    }

    private String _getString(Node rootNode, String key)
    {
        String value = null;
        try
        {
            value = getTextValue(_getNode(rootNode, key));
        }
        catch (Exception e)
        {
            getLogger().error(name + ": error while locating resource: " + key, e);
        }
        return value;
    }

    private static String getTextValue(Node element)
    {
        if (element == null) return null;
        NodeList list = element.getChildNodes();
        int listsize = list.getLength();

        Node item = null;
        String itemValue = null;

        for(int i = 0; i < listsize; i++)
        {
            item = list.item(i); if(item.getNodeType() != Node.TEXT_NODE) return null;
            itemValue = item.getNodeValue(); if(itemValue == null) return null;
            itemValue = itemValue.trim(); if(itemValue.length() == 0) return null;
            return itemValue;
        }
        return null;
    }

    public Node getNode(String key)  throws MissingResourceException
    {
        Node node = _getNode(key);
        if (node == null)
            throw new MissingResourceException("Unable to locate resource: " + key, XMLResourceBundle.class.getName(), key);
        else
            return node;
    }

    public Node getNode(Node rootNode, String key)  throws MissingResourceException
    {
        Node node = _getNode(rootNode, key);
        if (node == null)
            throw new MissingResourceException("Unable to locate resource: " + key, XMLResourceBundle.class.getName(), key);
        else
            return node;
    }

    private Node _getNode(String key)
    {
        return _getNode(this.doc.getDocumentElement(), key);
    }

    private Node _getNode(Node rootNode, String key)
    {
        Node node = null;
        try
        {
            node = XPathAPI.selectSingleNode(rootNode, key);
        }
        catch (Exception e)
        {
            getLogger().error("Error while locating resource", e);
        }
        return node;
    }

    public String substitute(String value, HashMap dictionary)
    {
        if (value == null || dictionary == null) return value;

        StringBuffer result = new StringBuffer(value.length());
        int startPos = 0;
        int endPos = 0;
        int lastPos = value.length();
        Object varValue = "";
        String varKey = "", oldKey = "";
        while (endPos < lastPos)
        {
            startPos = endPos;
            endPos = value.indexOf('{', startPos);
            if (endPos == -1)
                endPos = lastPos;
            result.append(value.substring(startPos, endPos));
            if (endPos < lastPos)
                endPos++;
            if (endPos < lastPos)
            {
                startPos = endPos;
                endPos = value.indexOf('}', startPos);
                if (endPos == -1)
                    endPos = lastPos;
                oldKey = varKey;
                varKey = value.substring(startPos, endPos);
                if (!oldKey.equals(varKey))
                    varValue = dictionary.get(varKey);
                if (varValue != null)
                {
                    getLogger().debug("Substituting var: " + varKey + " --> " + varValue);
                    result.append(varValue);
                }
                else
                {
                    getLogger().warn(name + ": var not found: " + varKey);
                    result.append('{').append(varKey).append('}');
                }
                if (endPos < lastPos)
                    endPos++;
            }
        }
        return result.toString();
    }

}

