/*
 * Copyright (C) The Apache Software Foundation. All rights reserved.
 *
 * This software is published under the terms of the Apache Software License
 * version 1.1, a copy of which has been included with this distribution in
 * the LICENSE file.
 */
package org.apache.avalon.framework.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

/**
 * DirectedGraph is a acyclic Directed graph implementation.
 *
 * @author <a href="mailto:donaldp@apache.org">Peter Donald</a>
 */
public class DependencyGraph
{
    protected final HashMap  m_map               = new HashMap();
    protected boolean        m_allowCircularity  = true;

    public void setAllowCircularity( final boolean allowCircularity )
    {
        m_allowCircularity = allowCircularity;
    }

    public void add( final String name, final String[] dependencies )
    {
        m_map.put( name, new GraphNode( name, dependencies ) );
    }

    public void remove( final String name )
    {
        m_map.remove( name );
    }

    public Dependency[] getDependencyList( final String name )
        throws CircularDependencyException
    {
        final ArrayList list = new ArrayList();
        final Dependency dependency = new Dependency( name , null );
        list.add( dependency );

        if( null != m_map.get( name ) )
        {
            final ArrayList stack = new ArrayList();
            stack.add( name );
            buildDependencyList( name, list, new ArrayList(), stack );
        }

        return (Dependency[])list.toArray( new Dependency[ 0 ] );
    }

    protected void buildDependencyList( final String name,
                                        final ArrayList list,
                                        final ArrayList done,
                                        final ArrayList stack )
        throws CircularDependencyException
    {
        if( done.contains( name ) ) return;
        done.add( name );

        final GraphNode node = (GraphNode)m_map.get( name );
        if( null == node ) return;

        final String[] dependencies = node.getDependencies();
        for( int i = 0; i < dependencies.length; i++ )
        {
            if( stack.contains( dependencies[ i ] ) )
            {
                if( m_allowCircularity ) continue;
                else
                {
                    throw new CircularDependencyException( dependencies[ i ], name, stack );
                }
            }

            if( done.contains( dependencies[ i ] ) ) continue;

            final Dependency dependency = new Dependency( dependencies[ i ], name );
            list.add( dependency );

            stack.add( dependencies[ i ] );
            buildDependencyList( dependencies[ i ], list, done, stack );
            stack.remove( stack.size() - 1 );
        }
    }

    public final static class Dependency
    {
        protected final String       m_name;
        protected final String       m_requiredBy;

        protected Dependency( final String name, final String requiredBy )
        {
            m_name = name;
            m_requiredBy = requiredBy;
        }

        public String getName()
        {
            return m_name;
        }

        public String getRequiredBy()
        {
            return m_requiredBy;
        }

        public String toString()
        {
            return getName();
        }
    }

    protected final static class GraphNode
    {
        protected final String     m_name;
        protected final String[]   m_dependencies;

        protected GraphNode( final String name, final String[] dependencies )
        {
            m_name = name;
            m_dependencies = dependencies;
        }

        public String getName()
        {
            return m_name;
        }

        public String[] getDependencies()
        {
            return m_dependencies;
        }
    }
}
